/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    See also http://www.gnu.org
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_POSIXTHREADS
#include <pthread.h>

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#define MAX_THREADS 16
#define MAX_MUTEXES 64

static pthread_t thr [MAX_THREADS];
static pthread_mutex_t mut [MAX_MUTEXES], mut_templ;
static pthread_mutex_t mut_templ = PTHREAD_MUTEX_INITIALIZER;
static int nthreads;

int Thread_start (void* (*func)(void*), int n)
{
  if (n < 0 || n >= MAX_THREADS)
  	return -1;
  return pthread_create (&thr[n], 0, func, 0);
}

void Thread_blockallsigs()
{
/* Let USR1 and USR2 through, gtk needs em */
  sigset_t set;
  sigfillset (&set);
  sigdelset (&set, SIGUSR1);
  sigdelset (&set, SIGUSR2);
  pthread_sigmask (SIG_SETMASK, &set, 0);
}

void Thread_init_mutexes (int n)
{
  int i;
  for (i=0; i<n; i++)
  	mut [i] = mut_templ;
}

void Thread_mutex_lock (int n)
{
//g_print("Locking %d\n", n);
  pthread_mutex_lock (&(mut[n]));
}

void Thread_mutex_unlock (int n)
{
//g_print("Unlocking %d\n", n);
  pthread_mutex_unlock (&(mut[n]));
}

void Thread_yield()
{
  sched_yield();
}

void Thread_kill (int n, int sig)
{
  pthread_kill (thr[n], sig);
}

void Thread_join (int n)
{
  pthread_join (thr[n], 0);
}

void Thread_exit()
{
  pthread_exit (0);
}
#else /* not HAVE_POSIXTHREADS */

#endif
